package src;

import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.chart.*;
import javafx.scene.chart.XYChart.Series;
import javafx.scene.control.Button;
import javafx.scene.control.DatePicker;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.stage.Stage;

import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class DateFilter extends Filter {

    private QueryExecutor qry_exec;
    private String startVal;
    private String endVal;
    private Label metric;
    private DataViewScreen dvs;
    private String metricName;
    private FontSizeHandler fontSizeHandler;

    private final int BUTTONS_FONT_SIZE = 16;
    private final int LABEL_FONT_SIZE = 16;
    private final int HEADING_LABEL_FONT_SIZE = 18;

    public DateFilter(QueryExecutor qry_exec, Label metric, DataViewScreen dvs, String metricName, FontSizeHandler fontSizeHandler){
        this.qry_exec = qry_exec;
        this.metric = metric;
        this.dvs = dvs;
        this.metricName = metricName;
        this.fontSizeHandler = fontSizeHandler;
    }

    public void load (){
        Stage s = new Stage();

        BorderPane bp = new BorderPane();
        VBox datePickers = new VBox();
        HBox controls = new HBox();
        HBox startDate = new HBox();
        HBox endDate = new HBox();
        HBox title = new HBox();

        DatePicker start = new DatePicker();
        DatePicker end = new DatePicker();
        Button finish = new Button ("Finish");
        Button cancel = new Button ("Cancel");
        Label startLabel = new Label("Start Date:");
        Label endLabel = new Label("End Date:");

        start.valueProperty().addListener(
                (obVal, oldValue, newValue) -> {
                    if (end.getValue() != null)
                        finish.setDisable(false);
                }
        );

        end.valueProperty().addListener(
                (obVal, oldValue, newValue) -> {
                    if (start.getValue() != null)
                        finish.setDisable(false);
                }
        );

        finish.setDisable(true);
        finish.setOnAction(
                action -> {
                   setStart(start);
                   setEnd(end);
                   s.setScene(super.done());
                    try {
                        java.util.concurrent.TimeUnit.SECONDS.sleep(1);
                    } catch (InterruptedException e) {
                        System.err.println("Interrupted exception: "+e);
                    }
                    s.close();
                   filter();
                }
        );

        cancel.setOnAction(
                action -> {
                    tearDown();
                    s.close();
                }
        );

        startDate.setAlignment(Pos.CENTER);
        startDate.setSpacing(10);
        startDate.getChildren().addAll(startLabel, start);

        endDate.setAlignment(Pos.CENTER);
        endDate.setSpacing(10);
        endDate.getChildren().addAll(endLabel, end);

        controls.setAlignment(Pos.CENTER);
        controls.setPadding(new Insets(5));
        controls.setSpacing(50);
        controls.getChildren().addAll(cancel, finish);

        datePickers.setAlignment(Pos.CENTER);
        datePickers.setPadding(new Insets(5));
        datePickers.setSpacing(30);
        datePickers.getChildren().addAll(startDate,endDate);

        title.setAlignment(Pos.CENTER);
        Label top = new Label("Select dates");
        top.setPadding(new Insets(10));
        title.getChildren().add(top);

        bp.setBottom(controls);
        bp.setCenter(datePickers);
        bp.setTop(title);

        fontSizeHandler.resetFPUNodes();
        fontSizeHandler.addFPUNode(top, LABEL_FONT_SIZE);
        fontSizeHandler.addFPUNode(startLabel, BUTTONS_FONT_SIZE);
        fontSizeHandler.addFPUNode(endLabel, BUTTONS_FONT_SIZE);
        fontSizeHandler.addFPUNode(cancel, BUTTONS_FONT_SIZE);
        fontSizeHandler.addFPUNode(finish, BUTTONS_FONT_SIZE);

        fontSizeHandler.updateFontSize(fontSizeHandler.FILTER_POPUP);

        s.setTitle("Date Picker");
        s.setScene(new Scene(bp));
        s.setMinHeight(300);
        s.setMinWidth(300);
        s.sizeToScene();
        s.show();
    }

    public void filter () {
        // Update Label
        String currentVal = metric.getText();
        String[] valArray = currentVal.split(" ");
        String granularity = dvs.getGranularity();
        String option = "";

        if (currentVal.contains("Bounces"))
            option = dvs.getBounceRegister();

        List<List<String[]>> result = qry_exec.executeQuery(metricName, "date", startVal + "," + endVal, granularity, option);
        String filteredValue = result.get(0).get(0)[0];
        if (filteredValue == null)
            filteredValue = "0";

        int replace = 0;
        if (!currentVal.contains("Bounces")) {
            for (String val : valArray) {
                if (val.contains(":")) {
                    break;
                }
                replace++;
            }
        }
        else {
            replace = valArray.length -1;
            for (int i = valArray.length-1 ; i >=0; i--){
                if (valArray[i].contains(":")) {
                    break;
                }
                replace--;
            }
        }
        String updatedMetric = currentVal.replace(valArray[replace+1], filteredValue);

        metric.setText(updatedMetric);

        List<String[]> filteredValues = result.get(1);

        if (metricName.equals("Bounces")) {
            PieChart pieChart = updateBounces(Integer.valueOf(filteredValue));
            LineChart lineChart = updateBounceRate(filteredValues, granularity, option);
            dvs.setMultiChart(pieChart, lineChart);
        }
        else {
            Series data = new Series();
            data.setName(dvs.getLineChart().getTitle());

            //cumulative chart for total cost, standard for others
            if (metricName.equals("Total Cost")) {
                double cumulativeCost = 0;
                for (int i = 0; i < filteredValues.size(); i++) {
                    String[] record = filteredValues.get(i);
                    cumulativeCost = cumulativeCost + Double.parseDouble(record[1]);
                    data.getData().add(new XYChart.Data(record[0], cumulativeCost));


                }
            }
            else {
                for (int i = 0; i < filteredValues.size(); i++) {
                    String[] record = filteredValues.get(i);
                    data.getData().add(new XYChart.Data(record[0], Double.valueOf(record[1])));
                }
            }

            Axis x = new CategoryAxis();
            x.setLabel("Day");
            Axis y = new NumberAxis();
            y.setLabel(dvs.getLineChart().getYAxis().getLabel());
            LineChart linechart = new LineChart(x, y);
            linechart.getData().addAll(data);
            linechart.setTitle(dvs.getLineChart().getTitle());
            if (metricName.equals("Clicks")){
                PieChart pieChart = dvs.getPieChart();
                pieChart.getData().clear();

                List<String[]> ageValues = qry_exec.executeQuery("Clicks", "date", startVal+","+endVal, "age groups",2, "");
                for(String[] record : ageValues) {
                    pieChart.getData().add(new PieChart.Data(record[0], Integer.valueOf(record[1])));
                }
                dvs.setMultiChart(pieChart, linechart);
            } else
                dvs.setLineChart(linechart);
        }
        dvs.showFilterApplied("Date range: "+startVal+" to "+endVal);
    }

    private PieChart updateBounces(Integer bounces) {
        // Update pie chart
        PieChart pieChart = dvs.getPieChart();
        pieChart.getData().remove(0);
        pieChart.getData().add(new PieChart.Data("Bounce clicks", bounces));

        return pieChart;
    }

    private LineChart updateBounceRate(List<String[]> filteredValues, String granularity, String option){
        String bounceRate = qry_exec.executeQuery(metricName,"bounce rate",1, granularity, option).get(0)[0];
        //metric.setText(metric.getText()+"Bounce Rate: " + bounceRate+"\n");

        List<String[]> clicksPerDay = qry_exec.executeQuery("Clicks", granularity, "").get(1);
        Series metric = new Series();
        metric.setName("Bounce rate per day");

        int i =0;
        while(i<filteredValues.size()){
            double avg = (Double.parseDouble(filteredValues.get(i)[1])) / (Double.parseDouble(clicksPerDay.get(i)[1]));
            metric.getData().add(new XYChart.Data(filteredValues.get(i)[0], avg));
            i++;

        }
        ArrayList<Series> data = new ArrayList<Series>(){
            {
                add(metric);
            }
        };

        Axis xAxis = new CategoryAxis();
        Axis yAxis = new NumberAxis();
        xAxis.setLabel("Day");
        yAxis.setLabel("Bounce Rate");

        LineChart lineChart = new LineChart(xAxis, yAxis);
        lineChart.getData().addAll(data);
        lineChart.setTitle("Bounce rate per day");
        lineChart.setStyle(lineChart.getStyle() + "; -fx-font-size: " + DataViewScreen.CHART_TITLE_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue()) + ";");
        lineChart.getXAxis().tickLabelFontProperty().set(Font.font(DataViewScreen.CHART_TICK_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
        lineChart.getYAxis().tickLabelFontProperty().set(Font.font(DataViewScreen.CHART_TICK_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
        yAxis.setTickLabelFont(Font.font(DataViewScreen.AXIS_LABEL_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
        xAxis.setTickLabelFont(Font.font(DataViewScreen.AXIS_LABEL_FONT_SIZE + (fontSizeHandler.getSizeValue() * fontSizeHandler.getIncrementValue())));
        return lineChart;
    }

    public void tearDown(){
        this.startVal = null;
        this.endVal = null;
    }

    @Override
    public void setFilterValues(String[] filterValues) {
        startVal = filterValues[0];
        endVal = filterValues[1];
    }

    @Override
    public String getFilterName() {
        return "Date Range";
    }

    private void setStart(DatePicker start) {
        startVal = start.getValue().format(DateTimeFormatter.ISO_LOCAL_DATE);
    }

    private void setEnd(DatePicker end){
        endVal = end.getValue().format(DateTimeFormatter.ISO_LOCAL_DATE);
    }

    @Override
    public String getSelectedValue() {
        return startVal+","+endVal;
    }
}
