package src;

import javafx.scene.Node;
import javafx.scene.chart.BarChart;
import javafx.scene.chart.Chart;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.PieChart;
import javafx.scene.text.Font;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FontSizeHandler {

    private Map<Node, Integer> dataViewScreen;
    private Map<Node, Integer> fileSelectScreen;
    private Map<Node, Integer> filterPopup;

    // sizeValue: -1 = small, 0 = medium, 1 = large
    private int sizeValue;
    private final int INCREMENT_VALUE = 3;
    public final String DATA_VIEW_SCREEN = "DVS";
    public final String FILE_SELECT_SCREEN = "FSS";
    public final String FILTER_POPUP = "FPU";

    public FontSizeHandler() {
        dataViewScreen = new HashMap<>();
        fileSelectScreen = new HashMap<>();
        filterPopup = new HashMap<>();
        sizeValue = 0;
    }

    public void addDVSNode(Node node, int size) {
        dataViewScreen.put(node, size);
    }

    public void addFSSNode(Node node, int size) {
        fileSelectScreen.put(node, size);
    }

    public void addFPUNode(Node node, int size) { filterPopup.put(node, size); }

    public void resetFPUNodes() { filterPopup.clear(); }

    private void update(Map<Node, Integer> hashMap) {
        boolean casted;
        for (Node node : hashMap.keySet()) {
            int newSize = hashMap.get(node) + (INCREMENT_VALUE * sizeValue);
            casted = false;

            try {
                LineChart chart = (LineChart) node;
                chart.setStyle(chart.getStyle() + "; -fx-font-size: " + newSize + ";");
                int labelSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.AXIS_LABEL_FONT_SIZE);
                chart.getXAxis().setStyle(chart.getXAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
                chart.getYAxis().setStyle(chart.getYAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
                int tickSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.CHART_TICK_FONT_SIZE);
                chart.getXAxis().setTickLabelFont(Font.font(tickSize));
                chart.getXAxis().setTickLabelFont(Font.font(tickSize));

                casted = true;
            }
            catch (ClassCastException e) {
                // failed to cast to LineChart
            }
            try {
                PieChart chart = (PieChart) node;
                chart.setStyle(chart.getStyle() + "; -fx-font-size: " + newSize + ";");
//                int labelSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.AXIS_LABEL_FONT_SIZE);
//                chart.getXAxis().setStyle(chart.getXAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
//                chart.getYAxis().setStyle(chart.getYAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
//                int tickSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.CHART_TICK_FONT_SIZE);
//                chart.getXAxis().setTickLabelFont(Font.font(tickSize));
//                chart.getXAxis().setTickLabelFont(Font.font(tickSize));
                casted = true;
            }
            catch (ClassCastException e) {
                // failed to cast to PieChart
            }
            try {
                BarChart chart = (BarChart) node;
                chart.setStyle(chart.getStyle() + "; -fx-font-size: " + newSize + ";");
                int labelSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.AXIS_LABEL_FONT_SIZE);
                chart.getXAxis().setStyle(chart.getXAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
                chart.getYAxis().setStyle(chart.getYAxis().getStyle() + "; -fx-font-size: " + labelSize + ";");
                int tickSize = newSize - (DataViewScreen.CHART_TITLE_FONT_SIZE - DataViewScreen.CHART_TICK_FONT_SIZE);
                chart.getXAxis().setTickLabelFont(Font.font(tickSize));
                chart.getXAxis().setTickLabelFont(Font.font(tickSize));

                casted = true;
            }
            catch (ClassCastException e) {
                // failed to cast to BarChart
            }

            if (!casted) {
                node.setStyle(node.getStyle() + "; -fx-font-size: " + newSize + ";");
            }
        }
    }

    public void updateFontSize(String updateClass) {
        if (updateClass.equals(DATA_VIEW_SCREEN)) {
            update(dataViewScreen);
        }
        else if (updateClass.equals(FILE_SELECT_SCREEN)) {
            update(fileSelectScreen);
        }
        else if (updateClass.equals(FILTER_POPUP)) {
            update(filterPopup);
        }
    }

    public void setSmallFontSize() {
        sizeValue = -1;
    }

    public void setMediumFontSize() {
        sizeValue = 0;
    }

    public void setLargeFontSize() {
        sizeValue = 1;
    }

    public int getSizeValue() { return sizeValue; }

    public int getIncrementValue() { return INCREMENT_VALUE; }
}
