package src;

import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.List;

public class MenuBarHandler {

    private Stage stage;
    private FontSizeHandler fontSizeHandler;
    private WindowsHandler windowsHandler;

    private List<Stage> openWindows = new ArrayList<>();

    public MenuBarHandler(Stage stage, FontSizeHandler fontSizeHandler, WindowsHandler windowsHandler) {
        this.stage = stage;
        this.fontSizeHandler = fontSizeHandler;
        this.windowsHandler = windowsHandler;
    }

    public MenuBar dataViewScreen(DatabaseController dbc, Scene fileSelectScene, QueryExecutor queryExecutor) {
        MenuBar menuBar = new MenuBar();

        Menu file = new Menu("File");
        MenuItem back = new MenuItem("Back to File Selection");
        MenuItem newWindow =  new MenuItem("Open New Window");

        back.setOnAction(e -> { windowsHandler.closeAllWindows(); Main.openFileSelectScreen(stage); });
        newWindow.setOnAction(e -> windowsHandler.openNewWindow(dbc, queryExecutor, fontSizeHandler, fileSelectScene));

        SeparatorMenuItem separator = new SeparatorMenuItem();
        MenuItem exit = new MenuItem("Exit");
        exit.setOnAction(e -> { System.exit(0); });

        Menu preferences = setupPreferences(fontSizeHandler.DATA_VIEW_SCREEN);

        file.getItems().addAll(back, newWindow, separator, exit);
        menuBar.getMenus().addAll(file, preferences);

        return menuBar;
    }

    public MenuBar fileSelectScreen() {
        MenuBar menuBar = new MenuBar();

        Menu file = new Menu("File");
        MenuItem exit = new MenuItem("Exit");
        exit.setOnAction(e -> { System.exit(0); });

        file.getItems().addAll(exit);

        Menu preferences = setupPreferences(fontSizeHandler.FILE_SELECT_SCREEN);

        menuBar.getMenus().addAll(file, preferences);

        return menuBar;
    }

    public Menu setupPreferences(String className) {
        Menu preferences = new Menu("Preferences");

        Menu fontSize = new Menu("Font Size");
        MenuItem small = new MenuItem("Small");
        MenuItem medium = new MenuItem("Medium");
        MenuItem large = new MenuItem("Large");

        small.setOnAction(e -> { fontSizeHandler.setSmallFontSize(); fontSizeHandler.updateFontSize(className); });
        medium.setOnAction(e -> { fontSizeHandler.setMediumFontSize(); fontSizeHandler.updateFontSize(className); });
        large.setOnAction(e -> { fontSizeHandler.setLargeFontSize(); fontSizeHandler.updateFontSize(className); });

        fontSize.getItems().addAll(small, medium, large);
        preferences.getItems().addAll(fontSize);
        return preferences;
    }
}