package src;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.chart.*;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class MetricEventsHandler {

    private String inUse = null;

    private FontSizeHandler fontSizeHandler;
    private QueryExecutor queryExecutor;
    private ComboBox<String> filtersComboBox;
    private ComboBox<String> granularityComboBox;
    private VBox centreVBox;
    private Label metricCount;
    private DataViewScreen dvs;

    public MetricEventsHandler(FontSizeHandler fontSizeHandler, QueryExecutor queryExecutor, ComboBox<String> filtersComboBox, ComboBox<String> granularityComboBox, VBox centreVBox, Label metricCount, DataViewScreen dvs) {
        this.fontSizeHandler = fontSizeHandler;
        this.queryExecutor = queryExecutor;
        this.filtersComboBox = filtersComboBox;
        this.granularityComboBox = granularityComboBox;
        this.centreVBox = centreVBox;
        this.metricCount = metricCount;
        this.dvs = dvs;
    }

    public LineChart buildLineChart(List<XYChart.Series> series, Axis x, Axis y, String title){
        LineChart chart = new LineChart(x, y);
        chart.getData().addAll(series);
        chart.setTitle(title);
        fontSizeHandler.addDVSNode(chart, DataViewScreen.CHART_TITLE_FONT_SIZE);
        return chart;
    }

    private PieChart buildPieChart(ArrayList<PieChart.Data> data, String title) {
        ObservableList<PieChart.Data> pieChartData = FXCollections.observableArrayList(data);
        final PieChart pieChart = new PieChart(pieChartData);
        pieChart.setTitle(title);
        fontSizeHandler.addDVSNode(pieChart, DataViewScreen.CHART_TITLE_FONT_SIZE);
        return pieChart;
    }

    public void userClickEvent(Button metricButton) {
        //making filter vbox visible after cost distribution hides it
        dvs.getRightBorderPane().setVisible(true);

        String[] cases = DataViewScreen.METRIC_BUTTON_TEXTS;
        dvs.removeBounceRegister();
        if (metricButton.getText().equals(cases[0])) {
            impressionClickEvent();
        }
        else if (metricButton.getText().equals(cases[1])) {
            clicksClickEvent();
        }
        else if (metricButton.getText().equals(cases[2])) {
            uniqueUsersClickEvent();
        }
        else if (metricButton.getText().equals(cases[3])) {
            bouncesClickEvent();
        }
        else if (metricButton.getText().equals(cases[4])) {
            conversionsClickEvent();
        }
        else if (metricButton.getText().equals(cases[5])) {
            totalCostClickEvent();
        }
        else if (metricButton.getText().equals(cases[6])) {
            clickThroughRateClickEvent();
        }
        else if (metricButton.getText().equals(cases[7])) {
            costPerAcquisitionClickEvent();
        }
        else if (metricButton.getText().equals(cases[8])) {
            costPerClickClickEvent();
        }
        else if (metricButton.getText().equals(cases[9])) {
            costPerThousandImpressionsClickEvent();
        }
        else if (metricButton.getText().equals(cases[10])) {
            //dvs.getRightBorderPane().setVisible(false);
            dvs.getRightBorderPane().setDisable(true);
            clickCostDistClickEvent();
        }

    }

    ////////////////////////
    private void impressionClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        String impressionsCount = queryExecutor.executeQuery("Impressions", granularity, "").get(0).get(0)[0];

        XYChart.Series impressionsChart = new XYChart.Series();
        impressionsChart.setName("Impressions per day");
        List<String[]> impressionsQueryResult = queryExecutor.executeQuery("Impressions", granularity, "").get(1);

        int i = 0;
        while(i < impressionsQueryResult.size()){
            impressionsChart.getData().add(new XYChart.Data(impressionsQueryResult.get(i)[0], Integer.parseInt(impressionsQueryResult.get(i)[1])));
            i++;
        }
        List<XYChart.Series> series = new ArrayList<XYChart.Series>();
        series.add(impressionsChart);

        Axis xAxis = new CategoryAxis();
        Axis yAxis = new NumberAxis();
        xAxis.setLabel("Day");
        yAxis.setLabel("Number Of Impressions");

        LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Impressions Per Day");
        metricCount.setText("Impressions Count:\n " + impressionsCount);

        centreVBox.getChildren().clear();
        centreVBox.getChildren().add(lineChart);
        centreVBox.setVgrow(lineChart, Priority.ALWAYS);
    }

    private void clicksClickEvent() {
        filtersComboBox.getSelectionModel().select(0);

        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        String clicksCount = queryExecutor.executeQuery("Clicks", granularity, "").get(0).get(0)[0];

        int maleCount = Integer.parseInt(queryExecutor.executeQuery("Clicks", "gender","Male", granularity, "").get(0).get(0)[0]);
        int femaleCount = Integer.parseInt(queryExecutor.executeQuery("Clicks", "gender","Female", granularity, "").get(0).get(0)[0]);
        int totalClicks = maleCount + femaleCount;
        String malePercentage = "Male: " + maleCount * 100 / totalClicks + "%";
        String femalePercentage = "Female: " + femaleCount * 100 / totalClicks + "%";

        XYChart.Series clicksChart = new XYChart.Series();
        clicksChart.setName("Clicks per day");

        List<String[]> clicksQueryResult = queryExecutor.executeQuery("Clicks", granularity, "").get(1);
        int i =0;
        while(i < clicksQueryResult.size()){
            clicksChart.getData().add(new XYChart.Data(clicksQueryResult.get(i)[0], Integer.parseInt(clicksQueryResult.get(i)[1])));
            i++;

        }
        List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(clicksChart); }};

        Axis xAxis = new CategoryAxis();
        Axis yAxis = new NumberAxis();
        xAxis.setLabel("Day");
        yAxis.setLabel("Number of clicks");
        LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Clicks Per Day");

        metricCount.setText("Click Count: \n" + clicksCount + " \n\n" + malePercentage + "\n" + femalePercentage);
        centreVBox.getChildren().add(lineChart);
        centreVBox.setVgrow(lineChart, Priority.ALWAYS);

        inUse = "Clicks";
        Thread thread = new Thread(
                () -> {
                    List <String[]> results = queryExecutor.executeQuery("Clicks", "age groups", 2, granularity, "");
                    // Pie Chart
                    ArrayList<PieChart.Data> pieChartData = new ArrayList<PieChart.Data>() {
                        {
                            results.forEach(x ->
                                    add(new PieChart.Data(x[0], Integer.valueOf(x[1])))
                            );
                        }
                    };
                    PieChart pieChart = buildPieChart(pieChartData, "Clicks By Age Ranges");
                    if (inUse.equals("Clicks"))
                        Platform.runLater(
                                () ->{
                                    centreVBox.getChildren().clear();
                                    centreVBox.getChildren().addAll(lineChart, pieChart);
                                    centreVBox.setVgrow(lineChart, Priority.SOMETIMES);
                                    centreVBox.setVgrow(pieChart, Priority.SOMETIMES);
                                }
                        );

                });
        thread.start();
    }

    private void uniqueUsersClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        String uniqueUsersCount = queryExecutor.executeQuery("Unique Users", granularity, "").get(0).get(0)[0];

        XYChart.Series uniqueUsersChart = new XYChart.Series();
        uniqueUsersChart.setName("Unique users per day");
        List<String[]> uniqueUsersQueryResult = queryExecutor.executeQuery("Unique Users", granularity, "").get(1);
        for (int i = 0; i < uniqueUsersQueryResult.size(); i++) {
            uniqueUsersChart.getData().add(new XYChart.Data(uniqueUsersQueryResult.get(i)[0], Integer.parseInt(uniqueUsersQueryResult.get(i)[1])));
        }
        List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(uniqueUsersChart); }};

        Axis xAxis = new CategoryAxis();
        Axis yAxis = new NumberAxis();
        xAxis.setLabel("Day");
        yAxis.setLabel("Number Of Unique Users");
        LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Unique Users");

        metricCount.setText("Unique Users Count: " + uniqueUsersCount);
        centreVBox.getChildren().clear();
        centreVBox.getChildren().add(lineChart);
        centreVBox.setVgrow(lineChart, Priority.ALWAYS);
    }

    private void bouncesClickEvent() {
        dvs.removeBounceRegister();
        filtersComboBox.getSelectionModel().select(0);
        metricCount.setText("Calculating Bounces...\n\nCalculating Bounce Rate");
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        centreVBox.getChildren().clear();


        Runnable rateRunnable = () -> {
            System.err.println("I'm running rate...");
            String register = dvs.getBounceRegister();
            System.out.println("Bounce register: "+register);
            Double bounceRate;
            List<String[]> bouncesPerGranularity;
            List<String[]> clicksPerGranularity;

            if (dvs.getCurrentFilter() != null) {
                System.out.println(dvs.getCurrentFilter().getFilterName());
                System.out.println(dvs.getCurrentFilter().getSelectedValue());
                if (dvs.getCurrentFilter().getFilterName().equals("Date Range")) {
                    bounceRate = Double.valueOf(queryExecutor.executeQuery("Bounces", "date", dvs.getCurrentFilter().getSelectedValue(), "bounce rate", 1, register).get(0)[0]);
                    bouncesPerGranularity = queryExecutor.executeQuery("Bounces", "date", dvs.getCurrentFilter().getSelectedValue(), granularity, register).get(1);
                    clicksPerGranularity = queryExecutor.executeQuery("Clicks", "date", dvs.getCurrentFilter().getSelectedValue(), granularity, "").get(1);
                } else {
                    bounceRate = Double.valueOf(queryExecutor.executeQuery("Bounces", dvs.getCurrentFilter().getFilterName().toLowerCase(), dvs.getCurrentFilter().getSelectedValue(), "bounce rate", 1, register).get(0)[0]);
                    bouncesPerGranularity = queryExecutor.executeQuery("Bounces", dvs.getCurrentFilter().getFilterName().toLowerCase(), dvs.getCurrentFilter().getSelectedValue(), granularity, register).get(1);
                    clicksPerGranularity = queryExecutor.executeQuery("Clicks", dvs.getCurrentFilter().getFilterName().toLowerCase(), dvs.getCurrentFilter().getSelectedValue(), granularity, "").get(1);
                }
            } else {
                bounceRate = Double.valueOf(queryExecutor.executeQuery("Bounces", "bounce rate", 1, granularity, register).get(0)[0]);
                bouncesPerGranularity = queryExecutor.executeQuery("Bounces", granularity, register).get(1);
                clicksPerGranularity = queryExecutor.executeQuery("Clicks", granularity, "").get(1);
            }
            Platform.runLater(
                    () -> {
                        metricCount.setText("Bounce Rate: " + bounceRate);
                        XYChart.Series bounceRateChart = new XYChart.Series();
                        bounceRateChart.setName("Bounce rate per day");

                        int i =0;
                        while(i < bouncesPerGranularity.size()){
                            double avg = (Double.parseDouble(bouncesPerGranularity.get(i)[1])) / (Double.parseDouble(clicksPerGranularity.get(i)[1]));
                            bounceRateChart.getData().add(new XYChart.Data(bouncesPerGranularity.get(i)[0], avg));
                            i++;

                        }
                        List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(bounceRateChart); }};

                        Axis xAxis = new CategoryAxis();
                        Axis yAxis = new NumberAxis();
                        xAxis.setLabel("Day");
                        yAxis.setLabel("Bounce Rate");

                        LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Bounce rate per day");
                        Iterator it = centreVBox.getChildren().iterator();
                        while (it.hasNext()){
                            if (it.next().getClass() == LineChart.class)
                                it.remove();
                        }
                        centreVBox.getChildren().add(lineChart);
                        centreVBox.setVgrow(lineChart, Priority.SOMETIMES);
                    }
            );
        };

        Runnable bounceRunnable = () -> {
            System.err.println("I'm running bounce...");
            String register = dvs.getBounceRegister();
            System.out.println("Bounce register: "+register);
            Integer bouncesCount;
            Integer totalClicks;
            if (dvs.getCurrentFilter() != null) {
                System.out.println(dvs.getCurrentFilter().getFilterName());
                System.out.println(dvs.getCurrentFilter().getSelectedValue());
                if (dvs.getCurrentFilter().getFilterName().equals("Date Range")) {
                    bouncesCount = Integer.valueOf(queryExecutor.executeQuery("Bounces", "date", dvs.getCurrentFilter().getSelectedValue(), granularity, register).get(0).get(0)[0]);
                    totalClicks = Integer.valueOf(queryExecutor.executeQuery("Clicks", "date", dvs.getCurrentFilter().getSelectedValue(), granularity, "").get(0).get(0)[0]);
                }else{
                    bouncesCount = Integer.valueOf(queryExecutor.executeQuery("Bounces", dvs.getCurrentFilter().getFilterName().toLowerCase(), dvs.getCurrentFilter().getSelectedValue(), granularity, register).get(0).get(0)[0]);
                    totalClicks = Integer.valueOf(queryExecutor.executeQuery("Clicks", dvs.getCurrentFilter().getFilterName().toLowerCase(), dvs.getCurrentFilter().getSelectedValue(), granularity, "").get(0).get(0)[0]);
                }
            } else {
                bouncesCount  = Integer.valueOf(queryExecutor.executeQuery("Bounces", granularity, register).get(0).get(0)[0]);
                totalClicks = Integer.valueOf(queryExecutor.executeQuery("Clicks", granularity, "").get(0).get(0)[0]);
            }

            Platform.runLater(

                    () -> {
                        metricCount.setText(metricCount.getText() + "\n\n Number Of Bounces: " + bouncesCount);

                        // Pie Chart
                        ArrayList<PieChart.Data> data = new ArrayList<PieChart.Data>(){
                            {
                                add(new PieChart.Data("Bounce Clicks", bouncesCount));
                                add(new PieChart.Data("Normal Clicks", totalClicks - bouncesCount));
                            }
                        };
                        PieChart pieChart = buildPieChart(data, "Proportion Of Bounces");
                        Iterator it = centreVBox.getChildren().iterator();
                        while (it.hasNext()){
                            if (it.next().getClass() == PieChart.class)
                                it.remove();
                        }
                        centreVBox.getChildren().add(pieChart);
                        centreVBox.setVgrow(pieChart, Priority.SOMETIMES);
                    }
            );
        };
        // Bounce rate thread
        Thread rateThread = new Thread (rateRunnable);
        // Bounces thread
        Thread bounceThread = new Thread (bounceRunnable);
        try {
            ComboBox registerSelector = new ComboBox();
            registerSelector.getItems().addAll("visited", "time on website");
            registerSelector.getSelectionModel().select(0);
            registerSelector.valueProperty().addListener(
                    (ov, oldv, newv) -> {
                        Thread bounce = new Thread (bounceRunnable);
                        Thread rate = new Thread (rateRunnable);
                        try {
                            rate.start();
                            rate.join();
                            bounce.start();
                        } catch (InterruptedException e){
                            e.printStackTrace();
                        }
                    }
            );
            dvs.setBounceRegister(registerSelector);
            rateThread.start();
            rateThread.join();
            bounceThread.start();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private void conversionsClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();

        Thread thread = new Thread(() -> {
            List<List<String[]>> result = queryExecutor.executeQuery("Conversions", granularity, "");
            Platform.runLater(() -> {
                    metricCount.setText("Conversions: " + result.get(0).get(0)[0]);
                    XYChart.Series conversionsChart = new XYChart.Series();
                    conversionsChart.setName("Number of Conversions");

                    List<String[]> conversionsQueryResult = result.get(1);
                    for (int i = 0; i < conversionsQueryResult.size(); i++) {
                        conversionsChart.getData().add(new XYChart.Data(conversionsQueryResult.get(i)[0], Integer.parseInt(conversionsQueryResult.get(i)[1])));
                    }

                    List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(conversionsChart); }};


                    Axis xAxis = new CategoryAxis();
                    Axis yAxis = new NumberAxis();
                    xAxis.setLabel("Day");
                    yAxis.setLabel("Number Of Conversions");
                    LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Conversions");

                    centreVBox.getChildren().clear();
                    centreVBox.getChildren().add(lineChart);
                    centreVBox.setVgrow(lineChart, Priority.ALWAYS);
            });
        });
        thread.start();

        metricCount.setText("Not Yet Implemented");
    }

    private void totalCostClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        metricCount.setText("Calculating Total Cost...");

        Thread thread = new Thread(
                () -> {
                    List<List<String[]>> result = queryExecutor.executeQuery("Total Cost", granularity, "");
                    Platform.runLater(
                            () -> {
                                metricCount.setText("Total Cost:\n " + result.get(0).get(0)[0] + " p");
                                XYChart.Series totalCost = new XYChart.Series();
                                totalCost.setName("Total cost per day");

                                List<String[]> totalCostQueryResult = result.get(1);
                                double cumulativeCost = 0;
                                for (int i = 0; i < totalCostQueryResult.size(); i++) {
                                    cumulativeCost += Double.parseDouble(totalCostQueryResult.get(i)[1]);
                                    totalCost.getData().add(new XYChart.Data(totalCostQueryResult.get(i)[0], cumulativeCost));
                                }

                                List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(totalCost); }};

                                Axis xAxis = new CategoryAxis();
                                Axis yAxis = new NumberAxis();
                                xAxis.setLabel("Day");
                                yAxis.setLabel("cost in pence");
                                LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Total Cost");

                                centreVBox.getChildren().clear();
                                centreVBox.getChildren().add(lineChart);
                                centreVBox.setVgrow(lineChart, Priority.ALWAYS);
                            }
                    );
                }
        );
        thread.start();

    }

    private void clickThroughRateClickEvent() {
        inUse = "Click-Through-Rate";
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();

        metricCount.setText("Calculating Click-Through-Rate...");

        Thread thread = new Thread(
                () -> {
                    XYChart.Series metric = new XYChart.Series();
                    metric.setName("CTR over time");
                    List<List<String[]>> result = queryExecutor.executeQuery("Click-Through-Rate", granularity, "");
                    if (inUse.equals("Click-Through-Rate"))
                        Platform.runLater(
                                ()->{
                                    metricCount.setText("Click Through Rate:\n " + Double.parseDouble(result.get(0).get(0)[0]));
                                    for(String[] record : result.get(1)){
                                        metric.getData().add(new XYChart.Data(record[0], Double.parseDouble(record[1])));
                                    }
                                    List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(metric); }};
                                    Axis xAxis = new CategoryAxis();
                                    Axis yAxis = new NumberAxis();
                                    xAxis.setLabel("Day");
                                    yAxis.setLabel("Click Through Rate");
                                    LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Click Through Rate");

                                    centreVBox.getChildren().clear();
                                    centreVBox.getChildren().addAll(lineChart);
                                    centreVBox.setVgrow(lineChart, Priority.ALWAYS);
                                }
                        );
                }
        );
        thread.start();
    }

    private void costPerAcquisitionClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        inUse = "Cost-Per-Acquisition";
        metricCount.setText("Calculating cost-per-acquisition...");
        Thread thread = new Thread(
                () -> {
                    XYChart.Series metric = new XYChart.Series();
                    metric.setName("CPA");
                    List<List<String[]>> result = queryExecutor.executeQuery("Cost-Per-Acquisition", granularity, "");
                    if (inUse.equals("Cost-Per-Acquisition"))
                        Platform.runLater(
                                ()-> {
                                    metricCount.setText("Cost per Acquisition:\n " + result.get(0).get(0)[0]);

                                    for (String[] record : result.get(1)) {
                                        metric.getData().add(new XYChart.Data(record[0], Double.parseDouble(record[1])));
                                    }
                                    List<XYChart.Series> series = new ArrayList<XYChart.Series>() {{
                                        add(metric);
                                    }};

                                    Axis xAxis = new CategoryAxis();
                                    Axis yAxis = new NumberAxis();
                                    xAxis.setLabel("Day");
                                    yAxis.setLabel("CPA");
                                    LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Cost-per-Acquisition");

                                    centreVBox.getChildren().clear();
                                    centreVBox.getChildren().add(lineChart);
                                    centreVBox.setVgrow(lineChart, Priority.ALWAYS);
                                }
                        );
                }
        );
        thread.start();
    }

    private void costPerClickClickEvent() {
        inUse = "Cost-Per-Click";
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        metricCount.setText("Calculating Cost-Per-Click...");
        Thread thread = new Thread(
                () -> {
                    XYChart.Series costPerClickChart = new XYChart.Series();
                    costPerClickChart.setName("CPC over time");
                    List<List<String[]>> result = queryExecutor.executeQuery("Cost-Per-Click", granularity, "");
                    if (inUse.equals("Cost-Per-Click"))
                        Platform.runLater(
                                ()->{
                                    metricCount.setText("Cost-Per-Click:\n " + result.get(0).get(0)[0]);

                                    for(String[] record : result.get(1)){
                                        costPerClickChart.getData().add(new XYChart.Data(record[0], Double.parseDouble(record[1])));
                                    }

                                    List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(costPerClickChart); }};

                                    Axis xAxis = new CategoryAxis();
                                    Axis yAxis = new NumberAxis();
                                    xAxis.setLabel("Day");
                                    yAxis.setLabel("CPC (Pence)");
                                    LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Cost-Per-Click");

                                    centreVBox.getChildren().clear();
                                    centreVBox.getChildren().add(lineChart);
                                    centreVBox.setVgrow(lineChart, Priority.ALWAYS);
                                }
                        );
                }
        );
        thread.start();
    }

    private void costPerThousandImpressionsClickEvent() {
        inUse = "Cost-Per-Thousand Impressions";
        filtersComboBox.getSelectionModel().select(0);
        String granularity = granularityComboBox.getSelectionModel().getSelectedItem();
        metricCount.setText("Calculating Cost-Per-Thousand Impressions...");

        Thread thread = new Thread(
                () -> {
                    XYChart.Series costPerClickChart = new XYChart.Series();
                    costPerClickChart.setName("CPM over time");
                    List<List<String[]>> result = queryExecutor.executeQuery("Cost-Per-Thousand Impressions", granularity, "");
                    if (inUse.equals("Cost-Per-Thousand Impressions"))
                        Platform.runLater(
                                ()->{
                                    metricCount.setText("Cost-Per-Thousand Impressions:\n " + result.get(0).get(0)[0]);

                                    for(String[] record : result.get(1)){
                                        costPerClickChart.getData().add(new XYChart.Data(record[0], Double.parseDouble(record[1])));
                                    }

                                    List<XYChart.Series> series = new ArrayList<XYChart.Series>(){{ add(costPerClickChart); }};

                                    Axis xAxis = new CategoryAxis();
                                    Axis yAxis = new NumberAxis();
                                    xAxis.setLabel("Day");
                                    yAxis.setLabel("CPM (Pence)");
                                    LineChart lineChart = buildLineChart(series, xAxis, yAxis, "Cost-Per-Thousand Impressions");

                                    centreVBox.getChildren().clear();
                                    centreVBox.getChildren().add(lineChart);
                                    centreVBox.setVgrow(lineChart, Priority.ALWAYS);
                                }
                        );
                }
        );
        thread.start();
    }

    private void clickCostDistClickEvent() {
        filtersComboBox.getSelectionModel().select(0);
        metricCount.setText("Calculating Click Cost Distribution...");

        int categorySize = 3;

        Thread thread = new Thread(
                () -> {
                    // executing query to calculate distribution of costs
                    List<String[]> result = queryExecutor.executeQuery("Click Cost Distribution",2,Integer.toString(categorySize));
                    Platform.runLater(
                            () -> {
                                metricCount.setText("Click Cost Distribution");

                                CategoryAxis xAxis = new CategoryAxis();
                                NumberAxis yAxis = new NumberAxis();
                                xAxis.setLabel("Click Cost Range (Pence)");
                                yAxis.setLabel("Frequency");

                                XYChart.Series series = new XYChart.Series();
                                series.setName("Click Cost");

                                BarChart barChart = new BarChart(xAxis, yAxis);
                                barChart.setTitle("Click Cost Distribution");

                                barChart.setCategoryGap(0);
                                barChart.setBarGap(0);

                                // populating series with queried data
                                //
                                for (int i = 0; i < result.size(); i++) {
                                    series.getData().add(new XYChart.Data(
                                            // labelling bars
                                            (0 + (categorySize * i)) + " - " + (categorySize + (categorySize * i)),
                                            Integer.valueOf(result.get(i)[1])
                                    ));
                                }

                                barChart.getData().addAll(series);

                                fontSizeHandler.addDVSNode(barChart, DataViewScreen.CHART_TITLE_FONT_SIZE);

                                centreVBox.getChildren().clear();
                                centreVBox.getChildren().addAll(barChart);
                                centreVBox.setVgrow(barChart, Priority.ALWAYS);


                            }
                    );
                }
        );
        thread.start();
    }
}
